@@ -5,7 +5,14 @@ module Test.HUnit.Lang
 (
   Assertion,
   assertFailure,
-  performTestCase
+  performTestCase,
+#if defined(__GLASGOW_HASKELL__) || defined(__HUGS__)
+-- * Internals
+-- |
+-- /Note:/ This is not part of the public API!  It is exposed so that you can
+-- tinker with the internals of HUnit, but do not expect it to be stable!
+  HUnitFailure (..)
+#endif
 )
 where
 
@@ -26,6 +33,7 @@ import Data.List (isPrefixOf)
 import System.IO.Error (ioeGetErrorString, try)
 #endif
 
+import Control.DeepSeq
 
 
 -- Interfaces
@@ -70,7 +78,11 @@ data HUnitFailure = HUnitFailure String
     deriving Show
 
 hunitFailureTc :: TyCon
+#if MIN_VERSION_base(4,4,0)
+hunitFailureTc = mkTyCon3 "HUnit" "Test.HUnit.Lang" "HUnitFailure"
+#else
 hunitFailureTc = mkTyCon "HUnitFailure"
+#endif
 {-# NOINLINE hunitFailureTc #-}
  
 instance Typeable HUnitFailure where
@@ -78,7 +90,7 @@ instance Typeable HUnitFailure where
 #ifdef BASE4
 instance Exception HUnitFailure
 
-assertFailure msg = E.throwIO (HUnitFailure msg)
+assertFailure msg = msg `deepseq` E.throwIO (HUnitFailure msg)
 
 performTestCase action = 
     do action
@@ -95,7 +107,7 @@ performTestCase action =
 
        E.Handler (\e -> return $ Just (False, show (e :: E.SomeException)))]
 #else
-assertFailure msg = E.throwDyn (HUnitFailure msg)
+assertFailure msg = msg `deepseq` E.throwDyn (HUnitFailure msg)
 
 performTestCase action = 
     do r <- E.try action
@@ -112,7 +124,7 @@ hunitPrefix = "HUnit:"
 
 nhc98Prefix = "I/O error (user-defined), call to function `userError':\n  "
 
-assertFailure msg = ioError (userError (hunitPrefix ++ msg))
+assertFailure msg = msg `deepseq` ioError (userError (hunitPrefix ++ msg))
 
 performTestCase action = do r <- try action
                             case r of Right () -> return Nothing
